/*
This source file is part of ProceduralGenerator (https://sourceforge.net/projects/proceduralgene/)

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
the program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*/
#ifndef ___Pg_BufferElement___
#define ___Pg_BufferElement___

#include "Prerequisites.h"
#include <cstdint>

namespace ProceduralTextures
{
	/*!
	\author 	Sylvain DOREMUS
	\version	0.6.1.0
	\date		03/01/2011
	\brief		Buffer element description
	\remark		Describes usage and type of an element in a vertex buffer
	*/
	struct GeneratorAPI BufferElementDeclaration
	{
		//!\~english	Stream index
		uint32_t		m_uiStream;
		//!\~english	Element usage
		eELEMENT_USAGE	m_eUsage;
		//!\~english	Element type
		eELEMENT_TYPE	m_eDataType;
		//!\~english	Offset in stream
		std::size_t		m_uiOffset;
	};
	/**
	 *\brief		Function to initialise a \p BufferElementDeclaration
	 *\param[out]	p_declaration	The \p BufferElementDeclaration to initialise
	 *\param[in]	p_uiStream		The stream index of the buffer element
	 *\param[in]	p_eUsage		The usage of the buffer element
	 *\param[in]	p_eDataType		The buffer element data type
	 *\param[in]	p_uiOffset		The offset of the element in the buffer
	 */
	extern GeneratorAPI void Initialise( BufferElementDeclaration & p_declaration, unsigned int p_uiStream, eELEMENT_USAGE p_eUsage, eELEMENT_TYPE p_eDataType, size_t p_uiOffset = 0 );
	/*!
	\author 	Sylvain DOREMUS
	\version	0.6.1.0
	\date		03/01/2011
	\brief		Description of all elements in a vertex buffer
	*/
	class GeneratorAPI BufferDeclaration
	{
	public:
		//! Typedef over an array of \p BufferElementDeclaration
		typedef std::vector< BufferElementDeclaration > BufferElementDeclarationArray;
		//! Typedef over an iterator on \p BufferElementDeclarationArray
		typedef BufferElementDeclarationArray::iterator 		iterator;
		//! Typedef over a constant iterator on \p BufferElementDeclarationArray
		typedef BufferElementDeclarationArray::const_iterator 	const_iterator;

	protected:
		//! Element description array
		BufferElementDeclarationArray	m_arrayElements;
		//! Byte size of the element
		std::size_t						m_uiStride;

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_elements	The elements array
		 */
		template< size_t N >
		BufferDeclaration( const BufferElementDeclaration (& p_elements)[N] )
		{
			DoInitialise( p_elements, N );
		}
		/**
		 *\brief		Constructor
		 *\param[in]	p_pElements		The elements array
		 *\param[in]	p_uiNbElements	The count of elements
		 */
		BufferDeclaration( const BufferElementDeclaration * p_pElements, size_t p_uiNbElements);
		/**
		 *\brief	Destructor
		 */
		virtual ~BufferDeclaration();
		/**
		 *\brief		Retrieves an iterator to the begin of the elements array
		 *\return		An iterator over the beginning of the array
		 */
		inline iterator Begin() { return m_arrayElements.begin(); }
		/**
		 *\brief		Retrieves a constant iterator to the begin of the elements array
		 *\return		A constant iterator over the beginning of the array
		 */
		inline const_iterator Begin()const { return m_arrayElements.begin(); }
		/**
		 *\brief		Retrieves an iterator over the end of the elements array
		 *\return		An iterator over the end of the array
		 */
		inline iterator End() { return m_arrayElements.end(); }
		/**
		 *\brief		Retrieves a constant iterator over the end of the elements array
		 *\return		A constant iterator over the end of the array
		 */
		inline const_iterator End()const { return m_arrayElements.end(); }
		/**
		 *\brief		Retrieves the elements count
		 *\return		The elements count
		 */
		inline size_t Size()const { return m_arrayElements.size(); }
		/**
		 *\brief		Retrieves the total elements byte count
		 *\return		The byte count
		 */
		inline size_t GetStride()const { return m_uiStride; }

	private:
		void DoInitialise( const BufferElementDeclaration * p_pElements, size_t p_uiNbElements);
	};
	/*!
	\author 	Sylvain DOREMUS
	\version	0.6.1.0
	\date		03/01/2011
	\brief		Holds data of a vertex buffer elements group
	*/
	class GeneratorAPI BufferElement
	{
	protected:
		//! Typedef over a \p PointBase
		typedef PointBase element;
		//! Typedef over a pointer of \p element
		typedef element * element_ptr;
		//! Typedef over an array of \p element
		typedef std::vector<element> element_array;
		//! Typedef over an array of \p element_ptr
		typedef std::vector<element_ptr> element_ptr_array;
		//! The declaration, used to define every part of the buffer element
		BufferDeclaration * m_pDeclaration;
		//! The buffer, containing buffer values
		uint8_t * m_pBuffer;
		//! The array of elements, each of them pointing on a part of the buffer
		element_ptr_array m_arrayElements;
		//! The stride of this element
		size_t m_uiStride;
		//! Tells if this element owns it's buffer or not
		bool m_bOwnBuffer;

	public:
		/**
		 *\brief		Constructor
		 *\remark		This BufferElement owns it's buffer
		 *\param[in]	p_pDeclaration	The declaration
		 */
		BufferElement( BufferDeclaration * p_pDeclaration );
		/**
		 *\brief		Destructor, deletes the buffer if owned by this element
		 */
		virtual ~BufferElement();
		/**
		 *\brief		Copy constructor
		 *\remark		This BufferElement doesn't own it's buffer
		 *\param[in]	p_element	The element to copy
		 */
		BufferElement( const BufferElement & p_element );
		/**
		 *\brief		Assignement operator
		 *\remark		This BufferElement doesn't own it's buffer anymore
		 *\param[in]	p_element		The element to copy
		 */
		BufferElement & operator =( const BufferElement & p_bufferElement);
		/**
		 *\brief		Links the element values to the ones in parameter. The element no longer owns it's values
		 *\param[in]	p_pBuffer	The values buffer
		 */
		void LinkCoords( void * p_pBuffer);
		/**
		 *\brief		Unlinks the element values. The element now owns it's coords
		 */
		void UnlinkCoords();
		/**
		 *\brief		Gets a reference to an element at given index
		 *\param[in]	p_uiIndex		The index
		 */
		template <typename T> inline T & GetElement( size_t p_uiIndex ) { return static_cast<T &>( * m_arrayElements[p_uiIndex]); }
		/**
		 *\brief		Gets a constant reference to an element at given index
		 *\param[in]	p_uiIndex		The index
		 */
		template <typename T> inline T const & GetElement( size_t p_uiIndex )const { return static_cast<T &>( * m_arrayElements[p_uiIndex]); }
		/**
		 *\return		Gets the element stride (size of an element)
		 */
		inline size_t GetStride()const { return m_uiStride; }
		/**
		 *\return		A pointer to the buffer values
		 */
		inline uint8_t * ptr() { return m_pBuffer; }
		/**
		 *\return		A constant pointer to the buffer values
		 */
		inline const uint8_t * const_ptr()const { return m_pBuffer; }

	protected:
		void DoLink();
	};
}

#endif
