/*
This source file is part of ProceduralGenerator (https://sourceforge.net/projects/proceduralgene/)

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
the program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*/
#ifndef ___PG_ConfigPanel___
#define ___PG_ConfigPanel___

#include "ProceduralGenerator.h"

namespace ProceduralTextures
{
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Base class for specific control type parameters
	*/
	class GeneratorAPI SpecificsBase
	{
	public:
		//! The control type
		eCONTROL_TYPE	m_eCtrlType;

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_eCtrlType	The control type
		 */
		SpecificsBase( eCONTROL_TYPE p_eCtrlType )
			:	m_eCtrlType	( p_eCtrlType	)
		{
		}
		/**
		 *\brief		Destructor
		 */
		virtual ~SpecificsBase()
		{
		}
	};
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Specialisation of Specifics for wxStaticText
	*/
	template<> class GeneratorAPI Specifics< eCONTROL_TYPE_STATIC > : public SpecificsBase
	{
	public:
		//! The label
		wxString		m_strText;
		//! The static control
		wxStaticText *	m_pControl;

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_strText	The control label
		 */
		Specifics( wxString const & p_strText )
			:	SpecificsBase	( eCONTROL_TYPE_STATIC	)
			,	m_strText		( p_strText				)
			,	m_pControl		( NULL					)
		{
		}
	};
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Specialisation of Specifics for wxTextCtrl
	*/
	template<> class GeneratorAPI Specifics< eCONTROL_TYPE_EDIT > : public SpecificsBase
	{
	public:
		//! The initial caption
		wxString		m_strText;
		//! The text control
		wxTextCtrl *	m_pControl;

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_strText	The control initial caption
		 */
		Specifics( wxString const & p_strText )
			:	SpecificsBase	( eCONTROL_TYPE_EDIT	)
			,	m_strText		( p_strText				)
			,	m_pControl		( NULL					)
		{
		}
	};
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Specialisation of Specifics for wxSlider
	*/
	template<> class GeneratorAPI Specifics< eCONTROL_TYPE_SLIDER > : public SpecificsBase
	{
	public:
		//! The minimal value for the slider
		int			m_iMin;
		//! The maximal value for the slider
		int			m_iMax;
		//! The initial value for the slider
		int			m_iValue;
		//! The slider control
		wxSlider *	m_pControl;

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_iMin		The minimal value
		 *\param[in]	p_iMax		The maximal value
		 *\param[in]	p_iValue	The initial value
		 */
		Specifics( int p_iMin, int p_iMax, int p_iValue )
			:	SpecificsBase	( eCONTROL_TYPE_SLIDER	)
			,	m_iMin			( p_iMin				)
			,	m_iMax			( p_iMax				)
			,	m_iValue		( p_iValue				)
			,	m_pControl		( NULL					)
		{
		}
	};
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Specialisation of Specifics for wxComboBox
	*/
	template<> class GeneratorAPI Specifics< eCONTROL_TYPE_COMBO > : public SpecificsBase
	{
	public:
		//! The initial value of the combo box
		wxString		m_strValue;
		//! The possible values array
		wxArrayString	m_arrayValues;
		// The combo box control
		wxComboBox *	m_pControl;

	public:
		/**
		 *\brief		Constructor
		 */
		Specifics()
			:	SpecificsBase	( eCONTROL_TYPE_COMBO	)
			,	m_pControl		( NULL					)
		{
		}
		/**
		 *\brief		Constructor
		 *\param[in]	p_strValue		The initial value
		 *\param[in]	p_arrayValues	The possible values array
		 */
		Specifics( wxString const & p_strValue, wxArrayString const & p_arrayValues )
			:	SpecificsBase	( eCONTROL_TYPE_COMBO	)
			,	m_pControl		( NULL					)
		{
			Initialise( p_strValue, &p_arrayValues[0], p_arrayValues.size() );
		}
		/**
		 *\brief		Constructor
		 *\param[in]	p_strValue		The initial value
		 *\param[in]	p_arrayValues	The possible values array
		 */
		template< size_t N >
		Specifics( wxString const & p_strValue, wxString const (& p_arrayValues)[N] )
			:	SpecificsBase	( eCONTROL_TYPE_COMBO	)
			,	m_pControl		( NULL					)
		{
			Initialise( p_strValue, p_arrayValues, N );
		}
		/**
		 *\brief		Constructor
		 *\param[in]	p_strValue		The initial value
		 *\param[in]	p_arrayValues	The possible values array
		 *\param[in]	p_uiCount		The possible values count
		 */
		Specifics( wxString const & p_strValue, wxString const * p_arrayValues, size_t p_uiCount )
			:	SpecificsBase	( eCONTROL_TYPE_COMBO	)
			,	m_pControl		( NULL					)
		{
			Initialise( p_strValue, p_arrayValues, p_uiCount );
		}
		/**
		 *\brief		Initialises the combo strings
		 *\param[in]	p_strValue		The initial value
		 *\param[in]	p_arrayValues	The possible values array
		 */
		void Initialise( wxString const & p_strValue, wxArrayString const & p_arrayValues )
		{
			Initialise( p_strValue, &p_arrayValues[0], p_arrayValues.size() );
		}
		/**
		 *\brief		Initialises the combo strings
		 *\param[in]	p_strValue		The initial value
		 *\param[in]	p_arrayValues	The possible values array
		 */
		template< size_t N >
		void Initialise( wxString const & p_strValue, wxString const (& p_arrayValues)[N] )
		{
			Initialise( p_strValue, p_arrayValues, N );
		}
		/**
		 *\brief		Initialises the combo strings
		 *\param[in]	p_strValue		The initial value
		 *\param[in]	p_arrayValues	The possible values array
		 *\param[in]	p_uiCount		The possible values count
		 */
		void Initialise( wxString const & p_strValue, wxString const * p_arrayValues, size_t p_uiCount )
		{
			m_arrayValues.clear();
			m_strValue = p_strValue;
			m_arrayValues.insert( m_arrayValues.end(), &p_arrayValues[0], &p_arrayValues[p_uiCount] );
		}
	};
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Specialisation of Specifics for wxButton
	*/
	template<> class GeneratorAPI Specifics< eCONTROL_TYPE_BUTTON > : public SpecificsBase
	{
	public:
		//! The button caption
		wxString	m_strText;
		//! The button control
		wxButton *	m_pControl;

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_strText		The button caption
		 */
		Specifics( wxString const & p_strText )
			:	SpecificsBase( eCONTROL_TYPE_BUTTON )
			,	m_strText	( p_strText	)
			,	m_pControl	( NULL		)
		{
		}
	};
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		The description of a panel describing options available with a generator.
	\remark		Each generator that needs a config panel must derive this class
	*/
	class ConfigPanel : public wxPanel
	{
	private:
		//! The generator
		ProceduralGenerator *	m_pGenerator;
		//! Array connecting wxEventType and eEVENT_TYPE
		wxEventType				m_arrayTypes[eEVENT_TYPE_COUNT];

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_arrayTypes	The array of wxEventType, used to make the link between wxEventType and eEVENT_TYPE
		 *\param[in]	p_pGenerator	The generator
		 *\param[in]	p_id			The panel ID
		 *\param[in]	p_ptPosition	The panel position
		 *\param[in]	p_size			The panel dimensions
		 */
		ConfigPanel( wxEventType const (&p_arrayTypes)[eEVENT_TYPE_COUNT], ProceduralGenerator * p_pGenerator, wxWindow * p_pWindow, wxWindowID p_id, const wxPoint & p_ptPosition=wxDefaultPosition, const wxSize & p_size=wxDefaultSize)
			:   wxPanel( p_pWindow, p_id, p_ptPosition, p_size)
			,	m_pGenerator( p_pGenerator )
		{
			for( int i = 0 ; i < eEVENT_TYPE_COUNT ; i++ )
			{
				m_arrayTypes[i] = p_arrayTypes[i];
			}

			SetBackgroundColour( p_pWindow->GetBackgroundColour() );
			SetForegroundColour( p_pWindow->GetForegroundColour() );
			CreateSpecifics();
		}
		/**
		 *\brief		Destructor
		 */
	    virtual ~ConfigPanel()
		{
		}
		/**
		 *\brief		Function used to effectively create the panel's controls
		 */
		void CreateSpecifics()
		{
			std::for_each( m_pGenerator->BeginCtrls(), m_pGenerator->EndCtrls(), [&]( ControlInfos & p_ctrlInfos )
			{
				switch( p_ctrlInfos.m_eType )
				{
				case eCONTROL_TYPE_STATIC:	DoCreateStatic	( p_ctrlInfos );	break;
				case eCONTROL_TYPE_EDIT:	DoCreateEdit	( p_ctrlInfos );	break;
				case eCONTROL_TYPE_SLIDER:	DoCreateSlider	( p_ctrlInfos );	break;
				case eCONTROL_TYPE_COMBO:	DoCreateCombo	( p_ctrlInfos );	break;
				case eCONTROL_TYPE_BUTTON:	DoCreateButton	( p_ctrlInfos );	break;
				}
			});
		}

	private:
		void DoCreateStatic( ControlInfos & p_ctrlInfos )
		{
			wxString l_strText;
			Specifics< eCONTROL_TYPE_STATIC > * l_pSpec = NULL;

			if( p_ctrlInfos.m_pSpecific && p_ctrlInfos.m_pSpecific->m_eCtrlType == eCONTROL_TYPE_STATIC )
			{
				l_pSpec = static_cast< Specifics< eCONTROL_TYPE_STATIC > * >( p_ctrlInfos.m_pSpecific );
				l_strText = l_pSpec->m_strText;
				l_pSpec->m_pControl = new wxStaticText( this, p_ctrlInfos.m_iId, l_strText, p_ctrlInfos.m_ptPosition, p_ctrlInfos.m_szSize, p_ctrlInfos.m_iStyle );

				if( p_ctrlInfos.m_pfnFunction )
				{
					l_pSpec->m_pControl->Connect( p_ctrlInfos.m_iId, m_arrayTypes[p_ctrlInfos.m_eEventType], p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
				}

				if( ! p_ctrlInfos.m_bVisible )
				{
					l_pSpec->m_pControl->Hide();
				}
			}
		}
		void DoCreateEdit( ControlInfos & p_ctrlInfos )
		{
			wxString l_strText;
			Specifics< eCONTROL_TYPE_EDIT > * l_pSpec = NULL;

			if( p_ctrlInfos.m_pSpecific && p_ctrlInfos.m_pSpecific->m_eCtrlType == eCONTROL_TYPE_EDIT )
			{
				l_pSpec = static_cast< Specifics< eCONTROL_TYPE_EDIT > * >( p_ctrlInfos.m_pSpecific );
				l_strText = l_pSpec->m_strText;
				l_pSpec->m_pControl = new wxTextCtrl( this, p_ctrlInfos.m_iId, l_strText, p_ctrlInfos.m_ptPosition, p_ctrlInfos.m_szSize, p_ctrlInfos.m_iStyle );

				if( p_ctrlInfos.m_pfnFunction )
				{
					l_pSpec->m_pControl->Connect( p_ctrlInfos.m_iId, m_arrayTypes[p_ctrlInfos.m_eEventType], p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
				}

				if( ! p_ctrlInfos.m_bVisible )
				{
					l_pSpec->m_pControl->Hide();
				}
			}
		}
		void DoCreateSlider( ControlInfos & p_ctrlInfos )
		{
			int l_iMin = 0;
			int l_iMax = 100;
			int l_iVal = 0;
			Specifics< eCONTROL_TYPE_SLIDER > * l_pSpec = NULL;

			if( p_ctrlInfos.m_pSpecific && p_ctrlInfos.m_pSpecific->m_eCtrlType == eCONTROL_TYPE_SLIDER )
			{
				l_pSpec = static_cast< Specifics< eCONTROL_TYPE_SLIDER > * >( p_ctrlInfos.m_pSpecific );
				l_iMin = l_pSpec->m_iMin;
				l_iMax = l_pSpec->m_iMax;
				l_iVal = l_pSpec->m_iValue;
				l_pSpec->m_pControl = new wxSlider( this, p_ctrlInfos.m_iId, l_iVal, l_iMin, l_iMax, p_ctrlInfos.m_ptPosition, p_ctrlInfos.m_szSize, p_ctrlInfos.m_iStyle );

				if( p_ctrlInfos.m_pfnFunction )
				{
					l_pSpec->m_pControl->Connect( p_ctrlInfos.m_iId, m_arrayTypes[p_ctrlInfos.m_eEventType], p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
				}

				if( ! p_ctrlInfos.m_bVisible )
				{
					l_pSpec->m_pControl->Hide();
				}
			}
		}
		void DoCreateCombo( ControlInfos & p_ctrlInfos )
		{
			wxArrayString l_arrayValues;
			wxString l_strValue;
			Specifics< eCONTROL_TYPE_COMBO > * l_pSpec = NULL;

			if( p_ctrlInfos.m_pSpecific && p_ctrlInfos.m_pSpecific->m_eCtrlType == eCONTROL_TYPE_COMBO )
			{
				l_pSpec = static_cast< Specifics< eCONTROL_TYPE_COMBO > * >( p_ctrlInfos.m_pSpecific );
				l_arrayValues = l_pSpec->m_arrayValues;
				l_strValue = l_pSpec->m_strValue;
				l_pSpec->m_pControl = new wxComboBox( this, p_ctrlInfos.m_iId, l_strValue, p_ctrlInfos.m_ptPosition, p_ctrlInfos.m_szSize, l_arrayValues, p_ctrlInfos.m_iStyle );

				if( p_ctrlInfos.m_pfnFunction )
				{
					l_pSpec->m_pControl->Connect( p_ctrlInfos.m_iId, m_arrayTypes[p_ctrlInfos.m_eEventType], p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
				}

				if( ! p_ctrlInfos.m_bVisible )
				{
					l_pSpec->m_pControl->Hide();
				}
			}
		}
		void DoCreateButton( ControlInfos & p_ctrlInfos )
		{
			wxString l_strText;
			Specifics< eCONTROL_TYPE_BUTTON > * l_pSpec = NULL;

			if( p_ctrlInfos.m_pSpecific && p_ctrlInfos.m_pSpecific->m_eCtrlType == eCONTROL_TYPE_BUTTON )
			{
				l_pSpec = static_cast< Specifics< eCONTROL_TYPE_BUTTON > * >( p_ctrlInfos.m_pSpecific );
				l_strText = l_pSpec->m_strText;
				l_pSpec->m_pControl = new wxButton( this, p_ctrlInfos.m_iId, l_strText, p_ctrlInfos.m_ptPosition, p_ctrlInfos.m_szSize, p_ctrlInfos.m_iStyle );

				if( p_ctrlInfos.m_pfnFunction )
				{
					if( p_ctrlInfos.m_eEventType == eEVENT_TYPE_LOAD_TEXT_FILEA )
					{
						Connect( p_ctrlInfos.m_iId, m_arrayTypes[eEVENT_TYPE_BUTTON_CLIKED], wxCommandEventHandler( ConfigPanel::OnLoadTextFileA ) );
						m_pGenerator->Connect( eEVENT_TEXTA, wxEVT_NULL, p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
					else if( p_ctrlInfos.m_eEventType == eEVENT_TYPE_LOAD_TEXT_FILEB )
					{
						Connect( p_ctrlInfos.m_iId, m_arrayTypes[eEVENT_TYPE_BUTTON_CLIKED], wxCommandEventHandler( ConfigPanel::OnLoadTextFileB ) );
						m_pGenerator->Connect( eEVENT_TEXTB, wxEVT_NULL, p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
					else if( p_ctrlInfos.m_eEventType == eEVENT_TYPE_LOAD_IMAGEA )
					{
						Connect( p_ctrlInfos.m_iId, m_arrayTypes[eEVENT_TYPE_BUTTON_CLIKED], wxCommandEventHandler( ConfigPanel::OnLoadImageA ) );
						m_pGenerator->Connect( eEVENT_IMAGEA, wxEVT_NULL, p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
					else if( p_ctrlInfos.m_eEventType == eEVENT_TYPE_LOAD_IMAGEB )
					{
						Connect( p_ctrlInfos.m_iId, m_arrayTypes[eEVENT_TYPE_BUTTON_CLIKED], wxCommandEventHandler( ConfigPanel::OnLoadImageB ) );
						m_pGenerator->Connect( eEVENT_IMAGEB, wxEVT_NULL, p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
					else if( p_ctrlInfos.m_eEventType == eEVENT_TYPE_LOAD_BINARY_FILEA )
					{
						Connect( p_ctrlInfos.m_iId, m_arrayTypes[eEVENT_TYPE_BUTTON_CLIKED], wxCommandEventHandler( ConfigPanel::OnLoadBinaryFileA ) );
						m_pGenerator->Connect( eEVENT_BINARYA, wxEVT_NULL, p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
					else if( p_ctrlInfos.m_eEventType == eEVENT_TYPE_LOAD_BINARY_FILEB )
					{
						Connect( p_ctrlInfos.m_iId, m_arrayTypes[eEVENT_TYPE_BUTTON_CLIKED], wxCommandEventHandler( ConfigPanel::OnLoadBinaryFileB ) );
						m_pGenerator->Connect( eEVENT_BINARYB, wxEVT_NULL, p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
					else if( p_ctrlInfos.m_eEventType == eEVENT_TYPE_LOAD_COLOURA )
					{
						Connect( p_ctrlInfos.m_iId, m_arrayTypes[eEVENT_TYPE_BUTTON_CLIKED], wxCommandEventHandler( ConfigPanel::OnLoadColourA ) );
						m_pGenerator->Connect( eEVENT_COLOURA, wxEVT_NULL, p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
					else if( p_ctrlInfos.m_eEventType == eEVENT_TYPE_LOAD_COLOURB )
					{
						Connect( p_ctrlInfos.m_iId, m_arrayTypes[eEVENT_TYPE_BUTTON_CLIKED], wxCommandEventHandler( ConfigPanel::OnLoadColourB ) );
						m_pGenerator->Connect( eEVENT_COLOURB, wxEVT_NULL, p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
					else
					{
						l_pSpec->m_pControl->Connect( p_ctrlInfos.m_iId, m_arrayTypes[p_ctrlInfos.m_eEventType], p_ctrlInfos.m_pfnFunction, NULL, m_pGenerator );
					}
				}

				if( ! p_ctrlInfos.m_bVisible )
				{
					l_pSpec->m_pControl->Hide();
				}
			}
		}
		void OnLoadImageA( wxCommandEvent & WXUNUSED( p_event ) )
		{
			wxString l_strWildcard = wxT( "All supported files (*.bmp;*.gif;*.png;*.jpg)|*.bmp;*.gif;*.png;*.jpg|BITMAP files (*.bmp)|*.bmp|GIF files(*.gif)|*.gif|JPEG files (*.jpg)|*.jpg|PNG files (*.png)|*.png");
			wxString l_strImagePath;
			wxFileDialog l_dialog( this, wxT( "Choose a picture" ), wxT( "./data" ), wxEmptyString, l_strWildcard );

			if( l_dialog.ShowModal() == wxID_OK )
			{
				l_strImagePath = l_dialog.GetPath();
				wxCommandEvent l_event( wxEVT_NULL, eEVENT_IMAGEA );
				l_event.SetString( l_strImagePath );
				m_pGenerator->AddPendingEvent( l_event );
			}
		}
		void OnLoadImageB( wxCommandEvent & WXUNUSED( p_event ) )
		{
			wxString l_strWildcard = wxT( "All supported files (*.bmp;*.gif;*.png;*.jpg)|*.bmp;*.gif;*.png;*.jpg|BITMAP files (*.bmp)|*.bmp|GIF files(*.gif)|*.gif|JPEG files (*.jpg)|*.jpg|PNG files (*.png)|*.png");
			wxString l_strImagePath;
			wxFileDialog l_dialog( this, wxT( "Choose a picture" ), wxT( "./data" ), wxEmptyString, l_strWildcard );

			if( l_dialog.ShowModal() == wxID_OK )
			{
				l_strImagePath = l_dialog.GetPath();
				wxCommandEvent l_event( wxEVT_NULL, eEVENT_IMAGEB );
				l_event.SetString( l_strImagePath );
				m_pGenerator->AddPendingEvent( l_event );
			}
		}
		void OnLoadTextFileA( wxCommandEvent & WXUNUSED( p_event ) )
		{
			wxString l_strFile;
			wxString l_strWildcard = wxT( "All files (*.*)|*.*");
			wxFileDialog l_dialog( this, wxT( "Choose a file" ), wxT( "./data" ), wxEmptyString, l_strWildcard );

			if( l_dialog.ShowModal() == wxID_OK )
			{
				l_strFile = l_dialog.GetPath();
				wxCommandEvent l_event( wxEVT_NULL, eEVENT_TEXTA );
				l_event.SetString( l_strFile );
				m_pGenerator->AddPendingEvent( l_event );
			}
		}
		void OnLoadTextFileB( wxCommandEvent & WXUNUSED( p_event ) )
		{
			wxString l_strFile;
			wxString l_strWildcard = wxT( "All files (*.*)|*.*");
			wxFileDialog l_dialog( this, wxT( "Choose a file" ), wxT( "./data" ), wxEmptyString, l_strWildcard );

			if( l_dialog.ShowModal() == wxID_OK )
			{
				l_strFile = l_dialog.GetPath();
				wxCommandEvent l_event( wxEVT_NULL, eEVENT_TEXTB );
				l_event.SetString( l_strFile );
				m_pGenerator->AddPendingEvent( l_event );
			}
		}
		void OnLoadBinaryFileA( wxCommandEvent & WXUNUSED( p_event ) )
		{
			wxString l_strFile;
			wxString l_strWildcard = wxT( "All files (*.*)|*.*");
			wxFileDialog l_dialog( this, wxT( "Choose a file" ), wxT( "./data" ), wxEmptyString, l_strWildcard );

			if( l_dialog.ShowModal() == wxID_OK )
			{
				l_strFile = l_dialog.GetPath();
				wxCommandEvent l_event( wxEVT_NULL, eEVENT_BINARYA );
				l_event.SetString( l_strFile );
				m_pGenerator->AddPendingEvent( l_event );
			}
		}
		void OnLoadBinaryFileB( wxCommandEvent & WXUNUSED( p_event ) )
		{
			wxString l_strFile;
			wxString l_strWildcard = wxT( "All files (*.*)|*.*");
			wxFileDialog l_dialog( this, wxT( "Choose a file" ), wxT( "./data" ), wxEmptyString, l_strWildcard );

			if( l_dialog.ShowModal() == wxID_OK )
			{
				l_strFile = l_dialog.GetPath();
				wxCommandEvent l_event( wxEVT_NULL, eEVENT_BINARYB );
				l_event.SetString( l_strFile );
				m_pGenerator->AddPendingEvent( l_event );
			}
		}
		void OnLoadColourA( wxCommandEvent & WXUNUSED( p_event ) )
		{
			wxColourDialog l_dialog( NULL );

			if (l_dialog.ShowModal() == wxID_OK)
			{
				wxColour l_colour = l_dialog.GetColourData().GetColour();
				wxCommandEvent l_event( wxEVT_NULL, eEVENT_COLOURA );
				l_event.SetInt( l_colour.GetPixel() );
				m_pGenerator->AddPendingEvent( l_event );
			}
		}
		void OnLoadColourB( wxCommandEvent & WXUNUSED( p_event ) )
		{
			wxColourDialog l_dialog( NULL );

			if (l_dialog.ShowModal() == wxID_OK)
			{
				wxColour l_colour = l_dialog.GetColourData().GetColour();
				wxCommandEvent l_event( wxEVT_NULL, eEVENT_COLOURB );
				l_event.SetInt( l_colour.GetPixel() );
				m_pGenerator->AddPendingEvent( l_event );
			}
		}
	};
}

#endif
