/*
This source file is part of ProceduralGenerator (https://sourceforge.net/projects/proceduralgene/)

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
the program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*/
#ifndef ___ProceduralGenerator___
#define ___ProceduralGenerator___

#include "Pixel.h"
#include "Plugin.h"
#include "GlObjects.h"

namespace ProceduralTextures
{
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Description of a control, in order to be able to create it completely
	\remark	It also holds an event and a callback function to call on reaction to the event
	*/
	class GeneratorAPI ControlInfos
	{
	public:
		//! The control type
		eCONTROL_TYPE			m_eType;
		//! The control ID
		wxWindowID				m_iId;
		//! The control position
		wxPoint					m_ptPosition;
		//! The control dimensions
		wxSize					m_szSize;
		//! Control's specific data
		SpecificsBase *			m_pSpecific;
		//! The control style
		int						m_iStyle;
		//! An event on which a function is called
		eEVENT_TYPE				m_eEventType;
		//! Function to use for specific event launched by the control
		wxObjectEventFunction	m_pfnFunction;
		//! Initial visibility status
		bool					m_bVisible;

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_eType			The control type
		 *\param[in]	p_iId			The control ID
		 *\param[in]	p_ptPosition	The control position
		 *\param[in]	p_szSize		The control dimensions
		 *\param[in]	p_pSpecific		Control's specific data
		 *\param[in]	p_iStyle		The control style
		 *\param[in]	p_event			An event on which a function is called
		 *\param[in]	p_pfnFunction	Function to use for specific event launched by the control
		 *\param[in]	p_bVisible		Initial visibility status
		 */
		ControlInfos( eCONTROL_TYPE p_eType, wxWindowID p_iId, wxPoint const & p_ptPosition, wxSize const & p_szSize, SpecificsBase * p_pSpecific, int p_iStyle=0, eEVENT_TYPE p_eEventType=eEVENT_TYPE_NONE, wxObjectEventFunction p_pfnFunction=NULL, bool p_bVisible=true );
		/**
		 *\brief		Copy constructor
		 *\param[in]	p_copy	Element to copy
		 */
		ControlInfos( ControlInfos const & p_copy );
		/**
		 *\brief		Move constructor
		 *\param[in]	p_copy	Element to move
		 */
		ControlInfos( ControlInfos && p_copy );
		/**
		 *\brief		Copy assignment operator
		 *\param[in]	p_copy	Element to copy
		 *\return		A reference over the current element
		 */
		ControlInfos & operator =( ControlInfos const & p_copy );
		/**
		 *\brief		Move assignment operator
		 *\param[in]	p_copy	Element to move
		 *\return		A reference over the current element
		 */
		ControlInfos & operator =( ControlInfos && p_copy );
		/**
		 *\brief		Destructor
		 */
		virtual ~ControlInfos();
	};
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Typedef on an array of ControlInfos
	*/
	typedef std::vector< ControlInfos > CtrlInfosArray;
	typedef std::vector< uint8_t > ByteArray;
	/*!
	\author		Sylvain DOREMUS
	\date		15/05/2013
	\brief		Class used to display
	*/
	class GeneratorAPI TextOverlay
	{
	private:
		class Glyph
		{
		private:
			wxPoint		m_ptPosition;
			wxSize		m_size;
			ByteArray	m_bitmap;

		public:
			Glyph();
			Glyph( const Glyph & p_glyph);
			Glyph & operator =( const Glyph & p_glyph);
			~Glyph();

			inline const wxSize &		GetSize		()const { return m_size; }
			inline const wxPoint &		GetPosition	()const { return m_ptPosition; }
			inline const ByteArray &	GetBitmap	()const { return m_bitmap; }

			inline void	SetBitmap	( const ByteArray & val )	{ m_bitmap = val; }
			inline void SetPosition	( const wxPoint & val )		{ m_ptPosition = val; }
			inline void SetSize		( const wxSize & val )		{ m_size = val; }
		};
		typedef std::vector< Glyph > GlyphArray;

	private:
		//! The OpenGl instance
		OpenGl * 				m_pOpenGl;
		//! The vertices buffer
		GlVertexBuffer			m_vtxBuffer;
		//! The indices buffer
		GlIndexBuffer			m_idxBuffer;
		//! The background texture, may be an image or a colour or nothing
		GlTexture				m_textureBackground;
		//! Tells if the background has been modified
		bool					m_bBackgroundModified;
		//! The foreground texture, may be an image or a colour or nothing
		GlTexture				m_textureForeground;
		//! Tells if the foreground has been modified
		bool					m_bForegroundModified;
		//! The currently displayed text texture
		GlTexture				m_textureText;
		//! The background texture with the text cut into it
		GlTexture				m_textureBackgroundCut;
		//! Frame buffer receiving the result of background - text
		GlFrameBuffer			m_frameBufferBackgroundCut;
		//! Texture receiving the result of (text * foreground)
		GlTexture				m_textureBlendedText;
		//! Frame buffer receiving the result of (text * foreground)
		GlFrameBuffer			m_frameBufferBlendedText;
		//! The texture bound to the result frame buffer
		GlTexture				m_textureResult;
		//! Frame buffer receiving the result of (background - text) + (text * foreground)
		GlFrameBuffer			m_frameBufferResult;
		//! The font glyphs
		GlyphArray				m_arrayGlyphs;
		//! The current overlay text
		wxString				m_strText;
		//! Tells if the text has been modified and needs redraw
		bool					m_bTextModified;
		//! The maximum height, for line ends
		int						m_iMaxHeight;
		//! Blend mode for text colour
		GlTexture::BlendMode	m_blendRgbTxt;
		//! Blend mode for text alpha
		GlTexture::BlendMode	m_blendAlphaTxt;
		//! Blend mode for background colour
		GlTexture::BlendMode	m_blendRgbImg;
		//! Blend mode for background alpha
		GlTexture::BlendMode	m_blendAlphaImg;
		//! The overlay dimensions
		wxSize					m_size;
		//! The overlay position
		wxPoint					m_position;
		//! Tells if the matrix has been modified and needs recomputing
		bool					m_bMatrixModified;
		//! Transformation matrix
		GLfloat					m_matrix[16];

	public:
		/**
		 *\brief		Constructor
		 *\param[in]	p_pOpenGl	The OpenGL instance
		 */
		TextOverlay( OpenGl * p_pOpenGl );
		/**
		 *\brief		Destructor
		 */
		~TextOverlay();
		/**
		 *\brief		All OpenGL global initialisations must be made in this function
		 */
		void Initialise();
		/**
		 *\brief		All OpenGL global cleanups must be made in this function
		 */
		void Cleanup();
		/**
		 *\brief		Retrieves font's glyph and initialises the overlay to given parameters
		 *\param[in]	p_iHeight	The font pixel height
		 *\param[in]	p_eStyle	The font style
		 *\param[in]	p_eWeight	The font weight
		 */
		void Create( int p_iHeight, wxFontStyle p_eStyle, wxFontWeight p_eWeight );
		/**
		 *\brief		Draws the text overlay into a given frame buffer
		 *\param[in]	p_frameBuffer	The frame buffer that receives the rendering
		 */
		void Draw( GlFrameBuffer & p_frameBuffer );
		/**
		 *\brief		Defines the text, tells it is modified
		 *\param[in]	p_strText	The new value
		 */
		inline void	SetText( const wxString & p_strText ) { m_strText = p_strText;m_bTextModified = true; }
		/**
		 *\brief		Defines the overlay size, sets it modified
		 *\param[in]	p_size	The new value
		 */
		inline void	SetSize( const wxSize & p_size ) { m_size = p_size;m_bMatrixModified = true; }
		/**
		 *\brief		Defines the overlay size, sets it modified
		 *\param[in]	p_size	The new value
		 */
		inline void	SetPosition( const wxPoint & p_position ) { m_position = p_position;m_bMatrixModified = true; }
		/**
		 *\brief		Defines the overlay background colour
		 *\param[in]	p_colour	The new value
		 */
		void SetBackground( wxColour const & p_colour );
		/**
		 *\brief		Defines the overlay foreground colour
		 *\param[in]	p_colour	The new value
		 */
		void SetForeground( wxColour const & p_colour );
		/**
		 *\brief		Defines the overlay background image
		 *\param[in]	p_image	The new value
		 */
		void SetBackground( wxImage const & p_image );
		/**
		 *\brief		Defines the overlay foreground image
		 *\param[in]	p_image	The new value
		 */
		void SetForeground( wxImage const & p_image );

	private:
		void DoSubRender();
	};
	typedef enum ePG_EVENT
	{//! Event to ask the main window to show a message box : p_event.GetString() may contain '||' then the left is the caption, the right is the message
		ePG_EVENT_SHOW_MESSAGE_BOX
	//! Event to ask the main window to resize to the given size  : wxSize( p_event.GetInt(), p_event..GetExtraLong() )
	,	ePG_EVENT_RESIZE_PARENT
	}	ePG_EVENT;
	/*!
	\author		Sylvain DOREMUS
	\date		23/05/2012
	\brief		Base class for every generator
	*/
	class GeneratorAPI ProceduralGenerator : public wxThread, public wxEvtHandler
	{
	protected:
		/*!
		\author		Sylvain DOREMUS
		\date		23/05/2012
		\brief		Worker thread
		\remark		A worker thread is defined to work on a vertical portion of the final surface.
					<br />This portion is defined by a top and a bottom
					<br />When the worker thread has finished it's job it sends an event to the generator
		*/
		class GeneratorAPI Thread : public wxThread
		{
		private:
			//! Tells the thread is stopped
			bool					m_bStopped;
			//! Tells the thread has been launched
			bool					m_bLaunched;

		protected:
			//! The mutex, mutable because it has no real influence on the thread status
			mutable wxMutex			m_mutex;
			//! The ProceduralGenerator
			ProceduralGenerator	*	m_pParent;
			//! The compute surface width
			int						m_iWidth;
			//! The compute surface height
			int						m_iHeight;
			//! The surface portion's top
			int						m_iTop;
			//! The surface portion's bottom
			int						m_iBottom;
			//! The pixel colour (image is in computed in white and m_pxColour)
			UbPixel					m_pxColour;
			//! The thread index
			size_t					m_uiIndex;

		public:
			/**
			 *\brief		Constructor
			 *\param[in]	p_pParent	The parent generator
			 *\param[in]	p_uiIndex	The thread index
			 *\param[in]	p_iWidth	The surface width
			 *\param[in]	p_iTop		The surface portion's top
			 *\param[in]	p_iBottom	The surface portion's bottom
			 *\param[in]	p_iHeight	The surface height
			 *\param[in]	p_pxColour	The pixel colour
			 */
			Thread( ProceduralGenerator * p_pParent, size_t p_uiIndex, int p_iWidth, int p_iTop, int p_iBottom, int p_iHeight, const UbPixel & p_pxColour );
			/**
			 *\brief		Destructor
			 */
			virtual ~Thread();
			/**
			 *\brief		The thread function
			 */
			virtual ExitCode Entry();
			/**
			 *\brief		The computing function
			 *\remark		To be implemented by derived classes
			 */
			virtual void Step()=0;
			/**
			 *\brief		Tells the thread is stopped or not
			 *\return		The stop status
			 */
			inline bool IsStopped()const { wxMutexLocker l_locker( m_mutex );return m_bStopped; }
			/**
			 *\brief		Stops the thread
			 */
			inline void Stop() { wxMutexLocker l_locker( m_mutex );m_bStopped = true; }
			/**
			 *\brief		Tells thread is launched or not
			 *\return		The launch status
			 */
			inline bool IsLaunched()const { wxMutexLocker l_locker( m_mutex );return m_bLaunched; }
			/**
			 *\brief		Launches the thread
			 */
			inline void Launch() { wxMutexLocker l_locker( m_mutex );m_bLaunched = true; }
			/**
			 *\brief		Sets the red value of the pixel
			 */
			inline void SetRed( uint8_t val ) { m_pxColour.r = val; }
			/**
			 *\brief		Sets the green value of the pixel
			 */
			inline void SetGreen( uint8_t val ) { m_pxColour.g = val; }
			/**
			 *\brief		Sets the blue value of the pixel
			 */
			inline void SetBlue( uint8_t val ) { m_pxColour.b = val; }
		};

	protected:
		wxSize						m_sizeDisplay;
		wxSize						m_sizeImage;
		unsigned long long			m_step;
		PixelBuffer					m_frontBuffer;
		PixelBuffer					m_backBuffer;
		UbPixel						m_pxColour;
		size_t						m_uiThreadCount;
		size_t						m_uiThreadEndedCount;
		bool						m_bStopped;
		wxFrame *					m_pMainFrame;
		bool						m_bNeedsConfigPanel;
		std::vector< Thread * >		m_arraySlaveThreads;
		bool						m_initialised;
		GlVertexBuffer				m_vertexBuffer;
		GlIndexBuffer				m_indexBuffer;
		GlTexture					m_texture;
		GlFrameBuffer				m_frameBuffer;
		GlTexture					m_textureOverlays;
		GlFrameBuffer				m_frameBufferOverlays;
		GlTexture					m_textureSave;
		GlFrameBuffer				m_frameBufferSave;
		bool						m_bToSave;
		mutable wxMutex				m_mutex;
		bool						m_bRendered;
		wxObjectEventFunction		m_pfnCpuStepEndHandler;
		CtrlInfosArray				m_arrayControls;
		wxLongLong					m_sleepTime;
		wxPoint						m_ptMousePosition;
		OpenGl						m_openGl;
		TextOverlay					m_overlayStatus;

	public:
		ProceduralGenerator( int p_width, int p_height, int p_iWndId, wxFrame * p_pMainFrame, bool p_bNeedsConfigPanel=true, int p_iSleepTime=10 );
		virtual ~ProceduralGenerator();

		virtual ExitCode Entry();

		void SetRed( uint8_t val );
		void SetGreen( uint8_t val );
		void SetBlue( uint8_t val );
		virtual void SwapBuffers();
		void InitialiseStep();
		void ClearAllThreads();
		void GlInitialise();
		void GlPreRender();
		void GlRender( bool p_bChanged);
		void GlPostRender();
		void GlCleanup();

		inline void SetCpuStepEndHandler( wxObjectEventFunction p_pfnHandler ) { m_pfnCpuStepEndHandler = p_pfnHandler; }

		void WaitForStepEnd();
		void Resize( int p_iWidth, int p_iHeight );
		void Resize( const wxSize & p_size );
		void SaveFrame();
		void ShowMessageBox( wxString const & p_title, wxString const & p_message, int p_flags = wxOK | wxCENTRE );

		template <class ThreadClass>
		ThreadClass * CreateThread( int WXUNUSED( iWidth ), int iTop, int iBottom, int iTotalHeight, const UbPixel & p_pxColour)
		{
			ThreadClass * l_pThread = new ThreadClass( this, m_arraySlaveThreads.size(), m_sizeImage.x, iTop, iBottom, iTotalHeight, p_pxColour);
			m_arraySlaveThreads.push_back( l_pThread);
			l_pThread->Create();
			return l_pThread;
		}

		inline uint8_t					GetRed				()const						{ return m_pxColour.r;														}
		inline uint8_t					GetGreen			()const						{ return m_pxColour.g;														}
		inline uint8_t					GetBlue				()const						{ return m_pxColour.b;														}
		inline bool						IsInitialised		()const						{ return m_initialised;														}
		inline const UbPixel *			GetPixels			()const						{ return m_frontBuffer.const_ptr();											}
		inline const PixelBuffer *		GetPixelsBuffer		()const						{ return & m_frontBuffer;													}
		inline PixelBuffer *			GetPixelsBuffer		()							{ return & m_frontBuffer;													}
		inline wxSize const &			GetDisplaySize		()const						{ return m_sizeDisplay;														}
		inline wxSize const &			GetImageSize		()const						{ return m_sizeImage;														}
		inline bool						NeedsConfigPanel	()const						{ return m_bNeedsConfigPanel;												}
		inline wxFrame *				GetMainFrame		()const						{ return m_pMainFrame;														}
		inline bool						IsRendered			()const						{ wxMutexLocker l_locker( m_mutex );return m_bRendered;						}
		inline void						SetRendered			( bool p_bVal )				{ wxMutexLocker l_locker( m_mutex );m_bRendered = p_bVal;					}
		inline bool						IsStopped			()const						{ wxMutexLocker l_locker( m_mutex );return m_bStopped;						}
		inline void						Stop				()							{ wxMutexLocker l_locker( m_mutex );m_bStopped = true;ClearAllThreads();	}
		inline CtrlInfosArray::iterator	BeginCtrls			()							{ return m_arrayControls.begin();											}
		inline CtrlInfosArray::iterator	EndCtrls			()							{ return m_arrayControls.end();												}
		inline void						SetDisplaySize		( const wxSize & p_size )	{ m_sizeDisplay = p_size;													}

	protected:
		/**
		 *\brief		Initialises the configuration panel structure
		 */
		virtual void DoGeneratePanel()=0;
		/**
		 *\brief		Resize things like frame buffers, render textures, internal buffers, ...
		 */
		virtual void DoResize( const wxSize & WXUNUSED( p_size ) ) {}
		/**
		 *\brief		Defines red value for generators that use it
		 */
		virtual void DoSetRed( uint8_t WXUNUSED( val ) ) {}
		/**
		 *\brief		Defines green value for generators that use it
		 */
		virtual void DoSetGreen( uint8_t WXUNUSED( val ) ) {}
		/**
		 *\brief		Defines blue value for generators that use it
		 */
		virtual void DoSetBlue( uint8_t WXUNUSED( val ) ) {}
		/**
		 *\brief		Thread specific step initialisation
		 */
		virtual void DoInitialiseStep() {}
		/**
		 *\brief		All OpenGL global initialisations must be made in this function
		 */
		virtual void DoGlInitialise()=0;
		/**
		 *\brief		All OpenGL frame initialisations must be made in this function
		 */
		virtual void DoGlPreRender()=0;
		/**
		 *\brief		All OpenGL draw calls must be made in this function
		 *\remark		Drawings will go in the generator frame buffer
		 */
		virtual void DoGlRender( bool & p_bChanged )=0;
		/**
		 *\brief		All OpenGL frame cleanups must be made in this function
		 */
		virtual void DoGlPostRender()=0;
		/**
		 *\brief		All OpenGL global cleanups must be made in this function
		 */
		virtual void DoGlCleanup()=0;

		void	DoCleanup	();
		wxSize	DoLoadImage	( const wxString & p_strImagePath, wxImage & p_pImage, PixelBuffer & p_buffer );
		void	DoSubRender	();
		void	DoSaveFrame	();

	private:
		void OnThreadEnd	( wxCommandEvent & p_event );
		void OnStop			( wxCommandEvent & p_event );
		void OnMouseMove	( wxMouseEvent & p_event );
	};
}

#endif
