/*
This source file is part of ProceduralGenerator (https://sourceforge.net/projects/proceduralgene/)

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU Lesser General Public License as published by the Free Software
Foundation; either version 2 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along with
the program; if not, write to the Free Software Foundation, Inc., 59 Temple
Place - Suite 330, Boston, MA 02111-1307, USA, or go to
http://www.gnu.org/copyleft/lesser.txt.
*/
#ifndef ___PG_Value___
#define ___PG_Value___

#include "Prerequisites.h"

// disable: "<type> needs to have dll-interface to be used by clients'
// Happens on STL member variables which are not public therefore is ok
#   pragma warning (disable : 4251)
// disable: "non dll-interface class used as base for dll-interface class"
// Happens when deriving from Singleton because bug in compiler ignores
// template export
#   pragma warning (disable : 4275)

namespace ProceduralTextures
{
	//! Functor used to delete an object with for_each
	template <class Object>
	struct ObjectDeleter
	{
		void operator()( Object * p_pPointer) { delete p_pPointer; }
	};
	//! Clears a container of it's objects
	template <class Container, class Object>
	void ClearContent( Container & p_container, Object p_object)
	{
	}
	//! Clears a container it's objects, which are deleted
	template <class Container, class Object>
	void ClearContent( Container & p_container, Object * p_object)
	{
		std::for_each( p_container.begin(), p_container.end(), ObjectDeleter<Object>());
	}
	//! Clears a container using previous functions
	template <class Container>
	void ClearContainer( Container & p_container)
	{
		typedef typename Container::value_type value_type;
		ClearContent( p_container, value_type());
		Container().swap( p_container);
	}
	//! Type of that structure exists if the condition is fullfilled
	template< bool condition, typename T=void  > struct enable_if
	{
	};
	//! Condition is fullfilled
	template< typename T > struct enable_if< true, T >
	{
		typedef T type;
	};
	//! Tells that T and U classes are of different types
	template<class T, class U> struct is_same
	{
		static const bool value = false;
	};
	//! Tells that T and T classes are of same types
	template<class T> struct is_same<T,T>
	{
		static const bool value = true;
	};
	//! Tells that T is a floating point basic type
	template<typename T> struct is_floating
	{
		static const bool value = (is_same<T, float>::value || is_same<T, double>::value || is_same<T, long double>::value);
	};
	//! Tells that T is an integral basic type
	template<typename T> struct is_integral
	{
		static const bool value = (is_same<T, int8_t>::value || is_same<T, int16_t>::value || is_same<T, int32_t>::value || is_same<T, int64_t>::value || is_same<T, uint8_t>::value || is_same<T, uint16_t>::value || is_same<T, uint32_t>::value || is_same<T, uint64_t>::value);
	};
	//! Tells that T is an unsigned basic type
	template<typename T> struct is_unsigned
	{
		static const bool value = (is_same<T, uint8_t>::value || is_same<T, uint16_t>::value || is_same<T, uint32_t>::value || is_same<T, uint64_t>::value);
	};
	//! tells if T is a type with a greater precision than U => floating has greater precision than non floating
	template <typename T, typename U> struct is_greater_precision_type
	{
		static const bool value = (is_floating<T>::value && ! is_floating<U>::value) || (sizeof( T) > sizeof( U) && ((is_floating<T>::value && is_floating<U>::value) || ( ! is_floating<T>::value && ! is_floating<U>::value)));
	};
	//! Struct used to select best way to put type in parameter : 'value' or 'const reference'
	template <typename T>
	struct CallTraits
	{
		template <typename U, bool Big> struct CallTraitsImpl;
		//! Struct used to select 'const reference' parameter type
		template <typename U>
		struct CallTraitsImpl<U, true>
		{
			typedef const U & Type;
		};
		//! Struct used to select 'value' parameter type
		template <typename U>
		struct CallTraitsImpl<U, false>
		{
			typedef U Type;
		};

		typedef typename CallTraitsImpl<T, (sizeof( T) > 8)>::Type ParamType;
	};

	template <typename T>
	T abs( const T & a)
	{
		return a < 0 ? -a : a;
	}

	template <typename T> class Policy
	{
	private:
		typedef T value_type;

	public:
		static value_type	zero			()							{ return 0; }
		static value_type	unit			()							{ return 1; }
		static void			init			( value_type & p_a)			{ p_a = zero(); }
		static bool			is_null			( const value_type & p_a)	{ return equals( p_a, zero()); }
		static value_type	negate			( const value_type & p_a)	{ return (is_unsigned<value_type>::value ? p_a : -p_a); }
		static value_type	ass_negate		( value_type & p_a)			{ return assign( p_a, negate( p_a)); }
		static value_type	stick			( value_type & p_a)
		{
			if (is_floating<value_type>::value)
			{
				if (is_null( p_a))
				{
					init( p_a);
				}
				else if (p_a > 0)
				{
					if (equals( p_a, float( int( p_a + 0.5f))))
					{
						assign( p_a, float( int( p_a + 0.5f)));
					}
				}
				else
				{
					if (equals( p_a, float( int( p_a - 0.5f))))
					{
						assign( p_a, float( int( p_a - 0.5f)));
					}
				}
			}
		}

		template <typename Ty> static value_type	convert			( const Ty & p_value)						{ return static_cast<value_type>( p_value); }
		template< typename Ty > static bool equals( const value_type & p_a, const Ty & p_b )
		{
			return substract( p_a, convert<Ty>( p_b)) <= std::numeric_limits<value_type>::epsilon();
		}
		template <typename Ty> static value_type	add				( const value_type & p_a, const Ty & p_b)	{ return p_a + convert<Ty>( p_b); }
		template <typename Ty> static value_type	substract		( const value_type & p_a, const Ty & p_b)	{ return p_a - convert<Ty>( p_b); }
		template <typename Ty> static value_type	multiply		( const value_type & p_a, const Ty & p_b)	{ return p_a * convert<Ty>( p_b); }
		template <typename Ty> static value_type	divide			( const value_type & p_a, const Ty & p_b)	{ return p_a / convert<Ty>( p_b); }
		template <typename Ty> static value_type	ass_add			( value_type & p_a, const Ty & p_b)			{ return assign( p_a, add<Ty>( p_a, p_b)); }
		template <typename Ty> static value_type	ass_substract	( value_type & p_a, const Ty & p_b)			{ return assign( p_a, substract<Ty>( p_a, p_b)); }
		template <typename Ty> static value_type	ass_multiply	( value_type & p_a, const Ty & p_b)			{ return assign( p_a, multiply<Ty>( p_a, p_b)); }
		template <typename Ty> static value_type	ass_divide		( value_type & p_a, const Ty & p_b)			{ return assign( p_a, divide<Ty>( p_a, p_b)); }
		template <typename Ty> static value_type	assign			( value_type & p_a, const Ty & p_b)			{ return p_a = convert<Ty>( p_b); }
	};
	//! 'bool' Policy specialisation
	template <> class Policy<bool>
	{
	private:
		typedef bool value_type;

	public:
		static value_type	zero			()							{ return false; }
		static value_type	unit			()							{ return true; }
		static void			init			( value_type & p_a)			{ p_a = zero(); }
		static bool			is_null			( const value_type & p_a)	{ return equals( p_a, zero()); }
		static value_type	negate			( const value_type & p_a)	{ return ! p_a; }
		static value_type	ass_negate		( value_type & p_a)			{ return assign( p_a, negate( p_a)); }
		static value_type	stick			( value_type & p_a)			{ return p_a; }

		template <typename Ty> static value_type	convert			( const Ty & p_value)						{ return p_value != 0;}
		template <typename Ty> static bool			equals			( const value_type & p_a, const Ty & p_b)	{ return p_a == convert<Ty>( p_b); }
		template <typename Ty> static value_type	add				( const value_type & p_a, const Ty & p_b)	{ return p_a || convert<Ty>( p_b); }
		template <typename Ty> static value_type	substract		( const value_type & p_a, const Ty & p_b)	{ return p_a || convert<Ty>( p_b); }
		template <typename Ty> static value_type	multiply		( const value_type & p_a, const Ty & p_b)	{ return p_a && convert<Ty>( p_b); }
		template <typename Ty> static value_type	divide			( const value_type & p_a, const Ty & p_b)	{ return p_a && convert<Ty>( p_b); }
		template <typename Ty> static value_type	ass_add			( value_type & p_a, const Ty & p_b)			{ return assign( p_a, add<Ty>( p_a, p_b)); }
		template <typename Ty> static value_type	ass_substract	( value_type & p_a, const Ty & p_b)			{ return assign( p_a, substract<Ty>( p_a, p_b)); }
		template <typename Ty> static value_type	ass_multiply	( value_type & p_a, const Ty & p_b)			{ return assign( p_a, multiply<Ty>( p_a, p_b)); }
		template <typename Ty> static value_type	ass_divide		( value_type & p_a, const Ty & p_b)			{ return assign( p_a, divide<Ty>( p_a, p_b)); }
		template <typename Ty> static value_type	assign			( value_type & p_a, const Ty & p_b)			{ return p_a = convert<Ty>( p_b); }
	};
}

#endif
