#include "ShaderEffects/GeneratorShaderEffects.h"

#include <cmath>

using namespace ProceduralTextures;

//*************************************************************************************************

Effect :: Effect( OpenGl * p_pOpenGl, wxImage * p_pImage )
	:	m_pImage			( p_pImage	)
	,	m_pShaderProgram	( NULL		)
	,	m_strVertexFile		(			)
	,	m_strFragmentFile	(			)
	,	m_bNewShader		( false		)
	,	m_llStartTime		( 0			)
	,	m_iWidth			( 0			)
	,	m_iHeight			( 0			)
	,	m_pOpenGl			( p_pOpenGl	)
{
	if( p_pImage && p_pImage->IsOk() )
	{
		m_iWidth	= p_pImage->GetWidth();
		m_iHeight	= p_pImage->GetHeight();
	}
}

Effect :: ~Effect()
{
	delete m_pShaderProgram;

	for (size_t i = 0 ; i < m_arrayImages.size() ; i++)
	{
		delete m_arrayImages[i];
	}

	m_arrayImages.clear();
	m_arrayImagesFiles.clear();
}

void Effect :: SetVertexFile( const wxString & p_strPath)
{
	m_strVertexFile = p_strPath;
}

void Effect :: SetFragmentFile( const wxString & p_strPath)
{
	m_strFragmentFile = p_strPath;
}

void Effect :: Compile()
{
	m_bNewShader = true;
}

void Effect :: SetImagePath( size_t p_iImage, const wxString & p_strImagePath)
{
	wxImage * l_pImage = NULL;

	if (p_iImage >= m_arrayImages.size())
	{
		l_pImage = new wxImage();
		m_arrayImages.push_back( l_pImage);
		m_arrayImagesFiles.push_back( p_strImagePath);
	}
	else
	{
		l_pImage = m_arrayImages[p_iImage];
		m_arrayImagesFiles[p_iImage] = p_strImagePath;
	}

	if (l_pImage->LoadFile( p_strImagePath))
	{
		int l_iWidth = l_pImage->GetWidth();
		int l_iHeight = l_pImage->GetHeight();
		std::cout << "Original : " << l_iWidth << "x" << l_iHeight << "\n";

		if (m_iWidth != l_iWidth || m_iHeight != l_iHeight)
		{
			std::cout << "Computed : " << m_iWidth << "x" << m_iHeight << "\n";
			l_pImage->Rescale( m_iWidth, m_iHeight, wxIMAGE_QUALITY_HIGH);
			std::cout << "Scaled : " << l_pImage->GetWidth() << "x" << l_pImage->GetHeight() << "\n";
		}
	}
	else
	{
//		ShowMessageBox( wxT( "Image loading error" ), wxT( "No handler found for this image format" ), wxOK | wxCENTRE | wxICON_ERROR );
	}
}

void Effect :: Initialise()
{
	if (m_pShaderProgram != NULL)
	{
		m_pShaderProgram->Cleanup();
	}
	else
	{
		m_pShaderProgram = new GlShaderProgram( m_pOpenGl );
	}

	if ( ! m_strVertexFile.empty())
	{
		m_pShaderProgram->CreateObject( eSHADER_OBJECT_TYPE_VERTEX);
		m_pShaderProgram->SetProgramFile( m_strVertexFile, eSHADER_OBJECT_TYPE_VERTEX);
	}

	if ( ! m_strFragmentFile.empty())
	{
		m_pShaderProgram->CreateObject( eSHADER_OBJECT_TYPE_PIXEL);
		m_pShaderProgram->SetProgramFile( m_strFragmentFile, eSHADER_OBJECT_TYPE_PIXEL);
	}

	m_pShaderProgram->Initialise();

	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_width"		),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_height"		),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_time"		),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_texture"		),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_modif1"		),	1 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_modif2"		),	2 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_modif3"		),	3 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_sep_type"	),	0 );
	m_pShaderProgram->CreateIntFrameVariable( wxT( "pg_sep_offset"	),	0 );

	m_llStartTime = wxGetLocalTimeMillis();
	m_bNewShader = false;
}

void Effect :: Cleanup()
{
	if (m_pShaderProgram != NULL)
	{
		m_pShaderProgram->Cleanup();
	}
}

bool Effect :: Activate( eSEPARATION p_eSeparationType, int p_iSeparationOffset)
{
	bool l_bReturn = true;

	if (m_pShaderProgram != NULL)
	{
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_width"),		m_iWidth);
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_height"),		m_iHeight);
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_sep_type"),	p_eSeparationType);
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_sep_offset"),	p_iSeparationOffset);
		m_pShaderProgram->SetIntFrameVariableValue( wxT( "pg_time"),		( wxGetLocalTimeMillis() - m_llStartTime).ToLong());
		m_pShaderProgram->Activate();
	}

	return l_bReturn;
}

void Effect :: Deactivate()
{
	if (m_pShaderProgram != NULL)
	{
		m_pShaderProgram->Deactivate();
	}
}

wxString Effect :: GetCompilerLog( eSHADER_OBJECT_TYPE p_eType)
{
	wxString l_strReturn;

	if (m_pShaderProgram != NULL)
	{
		GlShaderObject * l_pShader = m_pShaderProgram->GetProgram( p_eType);

		if (l_pShader != NULL)
		{
			l_strReturn = l_pShader->GetCompilerLog();
		}
	}

	return l_strReturn;
}

wxString Effect :: GetLinkerLog()
{
	wxString l_strReturn;

	if (m_pShaderProgram != NULL)
	{
		l_strReturn = m_pShaderProgram->GetLinkerLog();
	}

	return l_strReturn;
}

void Effect :: UpdateImages()
{
	m_iWidth = m_pImage->GetWidth();
	m_iHeight = m_pImage->GetHeight();

	for (size_t i = 0 ; i < m_arrayImages.size() ; i++)
	{
		m_arrayImages[i]->LoadFile( m_arrayImagesFiles[i]);
		m_arrayImages[i]->ResampleBicubic( m_iWidth, m_iHeight);
	}
}

//*************************************************************************************************

ShaderEffects :: ShaderEffects( int p_width, int p_height, int p_iWndId, wxFrame * p_pFrame)
	:	ProceduralGenerator			( p_width, p_height, p_iWndId, p_pFrame, true, 20				)
	,	m_iSeparationOffset			( 50															)
	,	m_eSeparationType			( eSEPARATION_NONE												)
	,	m_pSelectedEffect			( NULL															)
	,	m_specButtonImage			( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Image" ) )			)
	,	m_specStaticSeparator		( Specifics< eCONTROL_TYPE_STATIC	>( _( "Separation" ) )		)
	,	m_specComboSeparator		( Specifics< eCONTROL_TYPE_COMBO	>()							)
	,	m_specSliderOffset			( Specifics< eCONTROL_TYPE_SLIDER	>( 0, 100, 50 )				)
	,	m_specButtonReset			( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Reset Time" ) )		)
	,	m_specStaticShaders			( Specifics< eCONTROL_TYPE_STATIC	>( _( "Shaders" ) )			)
	,	m_specComboShaders			( Specifics< eCONTROL_TYPE_COMBO	>()							)
	,	m_specButtonVertexFile		( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Vertex" ) )			)
	,	m_specButtonFragmentFile	( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Fragment" ) )		)
	,	m_specButtonCompile			( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Compile" ) )			)
	,	m_specButtonCompilerLog		( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Compiler log" ) )	)
	,	m_specButtonRemove			( Specifics< eCONTROL_TYPE_BUTTON	>( _( "Remove" ) )			)
	,	m_frameBuffer1				( &m_openGl														)
	,	m_frameBuffer2				( &m_openGl														)
	,	m_texture1					( &m_openGl														)
	,	m_texture2					( &m_openGl														)
{
	wxString l_sepChoices[] =
	{	_( "Not separated"	)
	,	_( "Horizontally"	)
	,	_( "Vertically"		)
	};

	wxString l_shaChoices[] =
	{	_( "Shader 1"	)
	,	_( "New..."		)
	};

	m_specComboSeparator.Initialise( l_sepChoices[0], l_sepChoices );
	m_specComboShaders.Initialise( wxT( "" ), l_shaChoices );

	m_texture2.Resize( m_sizeImage);
	m_frameBuffer1.Resize( m_sizeImage);
	m_frameBuffer2.Resize( m_sizeImage);
	m_frameBuffer1.BindTexture( GL_COLOR_ATTACHMENT0, & m_texture1);
	m_frameBuffer2.BindTexture( GL_COLOR_ATTACHMENT0, & m_texture2);
	wxSystemOptions::SetOption( wxT( "msw.remap"), 0);
	wxInitAllImageHandlers();

	DoGeneratePanel();
}

ShaderEffects :: ~ShaderEffects()
{
	for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
	{
		delete m_arrayEffects[i];
	}

	m_arrayEffects.clear();
	wxImage::CleanUpHandlers();
	m_initialised = false;
	DoCleanup();
}

void ShaderEffects :: SetImagePath( const wxString & p_strImagePath)
{
	m_initialised = false;
	wxSize l_size = DoLoadImage( p_strImagePath, m_image, m_backBuffer);
	m_frontBuffer.init( Size( l_size.x, l_size.y ) );
	Resize( l_size);

	if (m_arrayEffects.size() == 0)
	{
		EffectAdd();
	}
	else
	{
		for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
		{
			m_arrayEffects[i]->UpdateImages();
		}
	}

	m_initialised = true;
}

void ShaderEffects :: ResetTime()
{
	for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
	{
		m_arrayEffects[i]->ResetTime();
	}
}

bool ShaderEffects :: EffectAdd()
{
	Effect * l_pEffect = new Effect( &m_openGl, &m_image );
	m_arrayEffects.push_back( l_pEffect );
	return true;
}

bool ShaderEffects :: EffectRemove( size_t p_uiIndex)
{
	bool l_bReturn = p_uiIndex < m_arrayEffects.size();

	if (l_bReturn)
	{
		Effect * l_pEffect = m_arrayEffects[p_uiIndex];
		std::vector <Effect *>::iterator l_it = m_arrayEffects.begin();
		m_arrayEffects.erase( l_it + p_uiIndex);
		delete l_pEffect;
	}

	return l_bReturn;
}

void ShaderEffects :: EffectSetVertexFile( size_t p_uiIndex, const wxString & p_strPath)
{
	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->SetVertexFile( p_strPath);
	}
}

void ShaderEffects :: EffectSetFragmentFile( size_t p_uiIndex, const wxString & p_strPath)
{
	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->SetFragmentFile( p_strPath);
	}
}

void ShaderEffects :: EffectSetImagePath( size_t p_uiIndex, size_t p_iImage, const wxString & p_strImagePath)
{
	m_initialised = false;

	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->SetImagePath( p_iImage, p_strImagePath);
	}

	m_initialised = true;
}

void ShaderEffects :: EffectCompile( size_t p_uiIndex)
{
	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->Compile();
	}
}

wxString ShaderEffects :: EffectGetCompilerLog( size_t p_uiIndex, eSHADER_OBJECT_TYPE p_eType)
{
	wxString l_strReturn;

	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->GetCompilerLog( p_eType);
	}

	return l_strReturn;
}

wxString ShaderEffects :: EffectGetLinkerLog( size_t p_uiIndex)
{
	wxString l_strReturn;

	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->GetLinkerLog();
	}

	return l_strReturn;
}

bool ShaderEffects :: EffectActivate( size_t p_uiIndex)
{
	bool l_bReturn = p_uiIndex < m_arrayEffects.size();

	if (l_bReturn)
	{
		l_bReturn = m_arrayEffects[p_uiIndex]->Activate( m_eSeparationType, m_iSeparationOffset);
	}

	return l_bReturn;
}

void ShaderEffects :: EffectDeactivate( size_t p_uiIndex)
{
	if (p_uiIndex < m_arrayEffects.size())
	{
		m_arrayEffects[p_uiIndex]->Deactivate();
	}
}

void ShaderEffects :: DoGlInitialise()
{
	m_texture1.Initialise();
	m_texture2.Initialise();

	m_frameBuffer1.Initialise();
	m_frameBuffer2.Initialise();
}

void ShaderEffects :: DoGlPreRender()
{
	m_texture1.Initialise( & m_backBuffer);
	m_texture2.Initialise( m_sizeImage);

	m_frameBuffer1.Initialise( m_sizeImage );
	m_frameBuffer2.Initialise( m_sizeImage );

	for( EffectPtrArray::iterator l_it = m_arrayEffects.begin() ; l_it != m_arrayEffects.end() ; ++l_it )
	{
		if( !(*l_it)->IsInitialised() )
		{
			(*l_it)->Initialise();
		}
	}

	m_openGl.Viewport( 0, 0, m_sizeImage.x, m_sizeImage.y);
	m_openGl.MatrixMode( GL_PROJECTION);
	m_openGl.LoadIdentity();
	m_openGl.Ortho( 0.0, 1.0, 0.0, 1.0, 0.0, 1.0);
	m_openGl.MatrixMode( GL_MODELVIEW);
	m_openGl.LoadIdentity();
}

void ShaderEffects :: DoGlRender( bool & p_bChanged)
{
	GlFrameBuffer * l_pFrameBufferTexture = & m_frameBuffer1;
	GlFrameBuffer * l_pFrameBufferRender = & m_frameBuffer2;

	m_texture1.Activate();
	m_texture1.UploadGlImage();
	m_texture1.Deactivate();

	for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
	{
		if (l_pFrameBufferRender->Activate())
		{
			if (EffectActivate( i))
			{
				m_openGl.Clear( GL_COLOR_BUFFER_BIT);
				l_pFrameBufferTexture->GetTexture( GL_COLOR_ATTACHMENT0)->Activate();
				DoSubRender();
				l_pFrameBufferTexture->GetTexture( GL_COLOR_ATTACHMENT0)->Deactivate();
				EffectDeactivate( i);
			}

			l_pFrameBufferRender->Deactivate();
		}

		std::swap( l_pFrameBufferTexture, l_pFrameBufferRender);
	}

	m_frameBuffer.Activate();
	m_openGl.Clear( GL_COLOR_BUFFER_BIT);
	l_pFrameBufferTexture->GetTexture( GL_COLOR_ATTACHMENT0)->Activate();
	DoSubRender();
	l_pFrameBufferTexture->GetTexture( GL_COLOR_ATTACHMENT0)->Deactivate();
	m_frameBuffer.Deactivate();
	p_bChanged = false;
}

void ShaderEffects :: DoGlPostRender()
{
}

void ShaderEffects :: DoGlCleanup()
{
	for (size_t i = 0 ; i < m_arrayEffects.size() ; i++)
	{
		m_arrayEffects[i]->Cleanup();
		delete m_arrayEffects[i];
	}

	m_arrayEffects.clear();

	m_frameBuffer1.Cleanup();
	m_frameBuffer2.Cleanup();

	m_texture1.Cleanup();
	m_texture2.Cleanup();
}

void ShaderEffects :: DoResize( const wxSize & WXUNUSED( p_size ) )
{
	m_frameBuffer1.Resize( m_sizeImage);
	m_frameBuffer2.Resize( m_sizeImage);
	m_texture1.Resize( m_sizeImage);
	m_texture2.Resize( m_sizeImage);
}

void ShaderEffects :: DoGeneratePanel()
{
	int iWidth = 150;
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_IMAGEPATH,		wxPoint(  10, wxDEFAULT_HEIGHT * 0	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonImage,			0,					eEVENT_TYPE_LOAD_IMAGEA,		wxCommandEventHandler(	ShaderEffects::OnImagePath			)			) );

	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_STATIC,	wxID_ANY,			wxPoint(  10, wxDEFAULT_HEIGHT * 1	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specStaticSeparator,		0,					eEVENT_TYPE_NONE,				NULL,															false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_COMBO,	eID_SEPTYPE,		wxPoint(  10, wxDEFAULT_HEIGHT * 2	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specComboSeparator,		wxCB_READONLY,		eEVENT_TYPE_COMBOBOX_SELECTED,	wxCommandEventHandler(	ShaderEffects::OnSepType			),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_SLIDER,	eID_SEPOFFSET,		wxPoint(  10, wxDEFAULT_HEIGHT * 3	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specSliderOffset,			wxSL_HORIZONTAL,	eEVENT_TYPE_SLIDER_UPDATED,		wxCommandEventHandler(	ShaderEffects::OnSepOffset			),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_RESETTIME,		wxPoint(  10, wxDEFAULT_HEIGHT * 4	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonReset,			0,					eEVENT_TYPE_BUTTON_CLIKED,		wxCommandEventHandler(	ShaderEffects::OnResetTime			),	false	) );

	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_STATIC,	wxID_ANY,			wxPoint(  10, wxDEFAULT_HEIGHT * 5	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specStaticShaders,		0,					eEVENT_TYPE_NONE,				NULL,															false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_COMBO,	eID_SHADERS,		wxPoint(  10, wxDEFAULT_HEIGHT * 6	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specComboShaders,			wxCB_READONLY,		eEVENT_TYPE_COMBOBOX_SELECTED,	wxCommandEventHandler(	ShaderEffects::OnSelectShader		),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_VERTEXFILE,		wxPoint(  10, wxDEFAULT_HEIGHT * 7	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonVertexFile,		0,					eEVENT_TYPE_LOAD_TEXT_FILEA,	wxCommandEventHandler(	ShaderEffects::OnVertexShaderPath	),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_FRAGMENTFILE,	wxPoint(  10, wxDEFAULT_HEIGHT * 8	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonFragmentFile,	0,					eEVENT_TYPE_LOAD_TEXT_FILEB,	wxCommandEventHandler(	ShaderEffects::OnFragmentShaderPath	),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_COMPILE,		wxPoint(  10, wxDEFAULT_HEIGHT * 9	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonCompile,		0,					eEVENT_TYPE_BUTTON_CLIKED,		wxCommandEventHandler(	ShaderEffects::OnShaderCompile		),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_COMPILERLOG,	wxPoint(  10, wxDEFAULT_HEIGHT * 10	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonCompilerLog,	0,					eEVENT_TYPE_BUTTON_CLIKED,		wxCommandEventHandler(	ShaderEffects::OnCompilerLog		),	false	) );
	m_arrayControls.push_back( ControlInfos( eCONTROL_TYPE_BUTTON,	eID_REMOVE,			wxPoint(  10, wxDEFAULT_HEIGHT * 11	), wxSize( iWidth -  20, wxDEFAULT_HEIGHT	), &m_specButtonRemove,			0,					eEVENT_TYPE_BUTTON_CLIKED,		wxCommandEventHandler(	ShaderEffects::OnRemove				),	false	) );
}

void ShaderEffects :: OnSepType( wxCommandEvent	& p_event)
{
	SetSeparationType( eSEPARATION( p_event.GetInt()));

	if (p_event.GetInt() != 0)
	{
		m_specSliderOffset.m_pControl->Show();
	}
	else
	{
		m_specSliderOffset.m_pControl->Hide();
	}
}

void ShaderEffects :: OnSepOffset( wxCommandEvent & p_event )
{
	SetSeparationOffset( p_event.GetInt());
}

void ShaderEffects :: OnResetTime( wxCommandEvent & WXUNUSED( p_event ) )
{
	ResetTime();
}

void ShaderEffects :: OnSelectShader( wxCommandEvent & p_event )
{
	unsigned int l_uiSelection = (unsigned int)p_event.GetSelection();

	if( l_uiSelection == m_specComboShaders.m_pControl->GetCount() - 1 )
	{
		if( EffectAdd() )
		{
			wxString l_strPage = wxT( "Shader ");
			l_strPage << m_specComboShaders.m_pControl->GetCount();
			m_specComboShaders.m_pControl->SetString( l_uiSelection, l_strPage );
			m_specComboShaders.m_pControl->Append( _( "New..." ) );
			m_specComboShaders.m_pControl->SetSelection( l_uiSelection );
		}
		else
		{
			l_uiSelection = m_specComboShaders.m_pControl->GetCount();
		}
	}
	else if( l_uiSelection > 0 )
	{
		m_specButtonRemove.m_pControl->Show();
	}
	else
	{
		m_specButtonRemove.m_pControl->Hide();
	}

	if( l_uiSelection < m_specComboShaders.m_pControl->GetCount() )
	{
		m_specButtonVertexFile.m_pControl->Show();
		m_specButtonFragmentFile.m_pControl->Show();
		m_pSelectedEffect = m_arrayEffects[l_uiSelection];
	}
	else
	{
		m_specButtonVertexFile.m_pControl->Hide();
		m_specButtonFragmentFile.m_pControl->Hide();
	}
}

void ShaderEffects :: OnRemove( wxCommandEvent & WXUNUSED( p_event ) )
{
	size_t l_uiToRemove = size_t( m_specComboShaders.m_pControl->GetSelection() );

	if( l_uiToRemove > 0 && l_uiToRemove < m_specComboShaders.m_pControl->GetCount() - 1 )
	{
		if( EffectRemove( l_uiToRemove ) )
		{
			m_specComboShaders.m_pControl->Delete( int( l_uiToRemove ) );
			m_specButtonVertexFile.m_pControl->Hide();
			m_specButtonFragmentFile.m_pControl->Hide();
			m_specButtonCompile.m_pControl->Hide();
			m_specButtonCompilerLog.m_pControl->Hide();
			m_specButtonRemove.m_pControl->Hide();
		}
	}

	m_pSelectedEffect = NULL;
}

void ShaderEffects :: OnShaderCompile( wxCommandEvent & WXUNUSED( p_event ) )
{
	if( m_pSelectedEffect )
	{
		m_pSelectedEffect->Compile();
		m_specButtonCompile.m_pControl->Hide();
		m_specButtonCompilerLog.m_pControl->Show();
	}
}

void ShaderEffects :: OnCompilerLog( wxCommandEvent & WXUNUSED( p_event ) )
{
	if( m_pSelectedEffect )
	{
		wxString l_strVertexLog		= m_pSelectedEffect->GetCompilerLog( eSHADER_OBJECT_TYPE_VERTEX	);
		wxString l_strFragmentLog	= m_pSelectedEffect->GetCompilerLog( eSHADER_OBJECT_TYPE_PIXEL	);
		wxString l_strLinkerLog		= m_pSelectedEffect->GetLinkerLog();
		wxString l_strText;

		if ( ! l_strVertexLog.empty())
		{
			l_strText += _( "Vertex Shader :" ) + wxT( "\n" ) + l_strVertexLog;
		}

		if ( ! l_strFragmentLog.empty())
		{
			if ( ! l_strText.empty())
			{
				l_strText += wxT( "\n\n" );
			}

			l_strText += _( "Fragment Shader :" ) + wxT( "\n" ) + l_strFragmentLog + wxT( "\n\n" );
		}

		if ( ! l_strLinkerLog.empty())
		{
			if ( ! l_strText.empty())
			{
				l_strText += wxT( "\n\n" );
			}

			l_strText += _( "Linker :" ) + wxT( "\n" ) + l_strLinkerLog + wxT( "\n\n" );
		}

		if (l_strText.empty())
		{
			l_strText = _( "No error" );
		}

		ShowMessageBox( _( "Compile and Link result" ), l_strText, wxOK | wxCENTRE | wxICON_INFORMATION );
	}
}

void ShaderEffects :: OnVertexShaderPath( wxCommandEvent & p_event )
{
	wxString l_strFile = p_event.GetString();

	if( m_pSelectedEffect && ! l_strFile.empty() )
	{
		wxString l_strFile = p_event.GetString();
		m_specButtonCompile.m_pControl->Show();
		m_specButtonCompilerLog.m_pControl->Hide();
		m_pSelectedEffect->SetVertexFile( l_strFile );
	}
}

void ShaderEffects :: OnFragmentShaderPath( wxCommandEvent & p_event )
{
	wxString l_strFile = p_event.GetString();

	if( m_pSelectedEffect && ! l_strFile.empty() )
	{
		wxString l_strFile = p_event.GetString();
		m_specButtonCompile.m_pControl->Show();
		m_specButtonCompilerLog.m_pControl->Hide();
		m_pSelectedEffect->SetFragmentFile( l_strFile );
	}
}

void ShaderEffects :: OnImagePath( wxCommandEvent & p_event )
{
	wxString l_strImagePath = p_event.GetString();

	if( ! l_strImagePath.empty() )
	{
		SetImagePath( l_strImagePath );
		m_specButtonReset.m_pControl->Show();
		m_specComboSeparator.m_pControl->Show();
		m_specComboShaders.m_pControl->Show();
	}
}

//******************************************************************************

IMPLEMENT_PLUGIN( SePlugin, ShaderEffects)

SePlugin :: SePlugin()
	:	Plugin< ShaderEffects, SePlugin >( wxT( "ShaderEffects" ) )
{
}

SePlugin :: ~SePlugin()
{
}

wxString SePlugin :: GetName()
{
	wxString l_strBaseName = _( "Shader Effects" );

#if !defined( NDEBUG )
	l_strBaseName += wxT( " (Debug)" );
#endif

	return l_strBaseName;
}

//*************************************************************************************************
